setwd(".")


# 1. Wgrywamy dane tygodniowe dla stp procentowych o rznych terminach zapadalnosci
################################################################################

rm(list = ls())
filename <- "dataUS.csv"
data     <- read.csv(filename, header=TRUE, sep=";", dec=".", row.names="daty")

# Uwaga: w pliku s stopy o ciaglej kapitalizacji
# r* = log(1+r/100)

daty     <- as.Date(rownames(data))
YC       <- data[,1:9]    # data for yield curves
MV       <- data[,10:11]  # data for macro variables

nazwyYC <- colnames(YC)
nazwyMV <- colnames(MV)

tau <- c(3,6,12,24,36,60,84,120, 360) # okres zapadalnosci w miesiacach

N <- ncol(YC)						# liczba terminow zapadalnosci
T <- nrow(YC)						# liczba obserwacji

# 2. Wykresy zmian stp w czasie oraz krzywych dochodowoci
######################################################################################

windows()
par(mfrow=c(2,2), cex = 0.7, bty="l")
# Wykres krzywych dochodowosci
plot(tau/12, YC[1,], main="Krzywa dochodowosci z 31 I 1980 r.", xlab="lata", ylab="" )
plot(tau/12, YC[T,], main="Krzywa dochodowosci z 28 XI 2014 r.", xlab="lata", ylab="" )
# Wykres ksztaltowania sie stop
plot(daty,YC[,1], main="stopy krotkie", type="l", ylab="", lty=1, col='black', ylim=c(0,16))
lines(daty,YC[,2], type="l", ylab="", lty=2, col='green')
lines(daty,YC[,3], type="l", ylab="", lty=3, col='red')
legend("topright", nazwyYC[1:3], lty=1:3, col=c('black','green','red'), bty="n")
plot(daty,YC[,4], main="stopy dlugie", type="l", ylab="", lty=1, col='black', , ylim=c(0,16))
lines(daty,YC[,6], type="l", ylab="", lty=2, col='green')
lines(daty,YC[,8], type="l", ylab="", lty=3, col='red')
legend("topright", nazwyYC[c(4,6,8)], lty=1:3, col=c('black','green','red'), bty="n")


# 3. Model Nelsona-Siegla oraz Svenssona dla dla pierwszej i ostatniej obserwacji     
#####################################################################################################
# zamiana na stopy o ciaglej kapitalizacji
library(YieldCurve)

# we need to change the class of our YC object to make the YieldCurve package work...
YC <- as.xts(YC)

# Nelson Siegel - oszacoweanie parametrow
NSPar <- Nelson.Siegel(rate=YC[c(1,T),], maturity=tau)
NSPar 

# Dopasowanie modelu do danych
NSfit1 <- NSrates(NSPar[1,],1:360)
NSfitT <- NSrates(NSPar[2,],1:360)

# Wykresy 
par(mfrow=c(1,2), cex = 0.7, bty="l")
plot(tau/12, YC[1,], main="Krzywa dochodowosci z 31 I 1980 r.", xlab="lata", ylab="" )
lines((1:360)/12,NSfit1, type="l", lty=1, col='black')
plot(tau/12, YC[T,], main="Krzywa dochodowosci z 28 XI 2014 r.", xlab="lata", ylab="" )
lines((1:360)/12,NSfitT, type="l", lty=1, col='black')

# Svensson - oszacowanie parametrow (potrzeba wicej punktw)
SPar <- Svensson(rate=YC[c(1,T),], maturity=tau)
SPar
Sfit1  <- Srates(SPar[1,], 1:360, whichRate = "Spot")
SfitT  <- Srates(SPar[2,], 1:360, whichRate = "Spot")

# Wykresy 
par(mfrow=c(1,2), cex = 0.7, bty="l")
plot(tau/12, YC[1,], main="Krzywa dochodowosci z 31 I 1980 r.", xlab="lata", ylab="" )
lines((1:360)/12,NSfit1, type="l", lty=1, col='black')
lines((1:360)/12,Sfit1, type="l", lty=2, col='red')
legend("topright", c("Nelson-Siegel","Svensson"), lty=1:2, col=c('black','red'), bty="n")
plot(tau/12, YC[T,], main="Krzywa dochodowosci z 28 XI 2014 r.", xlab="lata", ylab="" )
lines((1:360)/12,NSfitT, type="l", lty=1, col='black')
lines((1:360)/12,SfitT, type="l", lty=2, col='red')
legend("bottomright", c("Nelson-Siegel","Svensson"), lty=1:2, col=c('black','red'), bty="n")



# 4. Wykres dla dlugo- oraz krotkookresowej stopy procentowej 
##############################################################

# daty co 12 miesicy 
temp <- seq(1, T, 12)

NSPar  <- Nelson.Siegel(rate=YC[temp,], maturity=tau)
LTrate <- NSPar[,1]
STrate <- NSPar[,1]+NSPar[,2]

par(mfrow=c(1,2), cex = 0.7, bty="l")
plot(daty[temp],LTrate, main="Stopa dlugookresowa", type="l", ylab="", lty=1, col='black',ylim=c(0,16))
plot(daty[temp],STrate, main="Stopa krotkookresowa", type="l", ylab="", lty=1, col='black',ylim=c(0,16))

# save(NSPar, file = "NSPar.RData") 

# 5. Prognoza dla stp o zapadalnosci k (3M)
##############################################

# momenty dla ktrych dokonujemy prognozy 
datyF <- seq(from=daty[T]+30, to=as.Date("2017-12-31"), by=30)
# dla danych tygodniowych argument by=7

# ich odleglosc od ostatniej obserwacji (w mies.)
tauF  <- as.numeric(datyF-daty[T])*12/365.25
# liczba obserwacji
h     <- length(datyF)
# jaka stopa (3M, wybierz zapadalno ze zbioru tau)
k     <- 3

# szacujemy parametry modelu NS/Svensona
Par <- Nelson.Siegel(rate=YC[T,], tau)
#Par  <- Svensson(rate=YC[T,], tau)

# Stopy kasowe o terminach tauF oraz tauF+k
temp <- matrix(NA,h,2)
temp[,1] <- NSrates(Par, tauF)
temp[,2] <- NSrates(Par, tauF+k)
#temp[,1] <- Srates(Par, tauF,   whichRate = "Spot")
#temp[,2] <- Srates(Par, tauF+k, whichRate = "Spot")

# Prognoza dla dat datyF
Prognoza <- ((tauF+k)*temp[,2] - tauF*temp[,1])/k

# wgrywamy przeszle dane
Stopy <- YC[,which(tau==k)]

# wykres prognozy
par(mfrow=c(1,1), cex = 0.7, bty="l")
plot(daty, Stopy, type="l", lty=1, col='black', xlim=as.Date(c("2005-01-01","2017-12-31")), ylim=c(0,6), ylab="") 
lines(datyF,Prognoza, type="l", lty=2, col='red')
title("Prognoza stp proc. z krzywej dochodowosci ")


# Dodatek. Wykresy ksztaltu krzywych dochodowosci modelu Nelsona-Siegela 
# Replikacja wykresw z artykulu zrdlowego				 #	
####################################################################

# parametry

b0  <- 5
b1  <- -2
b2  <- seq(from = -6, to = 12, by = 3)
lam <- 1/12

# terminy zapadalnosci (miesiace)
tau    <- 0:120; tau[1] <- 0.00001					

# czynniki w modelu NS
LT <- rep(1,length(tau))			# czesc dlugookresowa
MT <- exp(-lam*tau)*(lam*tau)		# czesc sredniookresowa
ST <- exp(-lam*tau)				# czesc krotkookresowa

# stopy dla roznych b2 (rola czynnika srediookresowego)
y    <- matrix(NA,7,length(tau))
for (i in 1:7) y[i,] <- b0*LT + b1*ST + b2[i]*MT # ksztalt krzywych dla roznych b2


par(mfrow=c(1,2), cex = 0.7, bty="l", pin=c(2,2))
plot(tau/12,LT,xlab="lata",ylab="",ylim=c(0,1.2),main="Skladowe stopy terminowej", type="l")
lines(tau/12, MT, type="l")
lines(tau/12, ST, type="l")
text(6,1.1,"dlugi okres")
text(6.5, 0.2, expression(paste("sredni okres: ", tau~e^{-tau} )))
text(3, 0.65, expression(paste("krotki okres: ", e^{tau} )))

# wykres ksztaltow dla roznych wartosci b2
plot(tau/12,y[1,], type="l", xlab="lata", ylab="", main="Rola czynnika MT", ylim=c(0,10))
for (i in 2:7) lines(tau/12,y[i,], type="l")

