setwd(".")


# 1. Wgrywamy dane tygodniowe dla stp procentowych o rznych terminach zapadalnosci
################################################################################

rm(list = ls())
filename <- "dataUS.csv"
data     <- read.csv(filename, header=TRUE, sep=";", dec=".", row.names="daty")

# Uwaga: w pliku s stopy o ciaglej kapitalizacji
# r* = log(1+r/100)

daty     <- as.Date(rownames(data))

YC <- data[,1:9]    # data for yield curves
MV <- data[,10:11]  # data for macro variables

nazwyYC <- colnames(YC)
nazwyMV <- colnames(MV)

tau <- c(3,6,12,24,36,60,84,120,360) # okres zapadalnosci w miesiacach

N <- ncol(YC)						# liczba terminow zapadalnosci
T <- nrow(YC)						# liczba obserwacji
K <- length(tau)

# 2. Estymacja czynnikow modelu Nelsona-Siegla
######################################################################################
lam <- 0.0609;

LT  <- rep(1,K)			                                # czesc dlugookresowa
MT  <- (1-exp(-lam*tau))/(lam*tau) - exp(-lam*tau); # czesc sredniookresowa
ST  <- (1-exp(-lam*tau))/(lam*tau)				  # czesc krotkookresowa

x   <- cbind(LT,MT,ST)

beta <- solve(t(x)%*%x)%*%t(x)%*%t(YC)
beta <- t(beta)
L    <- beta[,1] # level factor
C    <- beta[,2] # curvature factor
S    <- beta[,3] # slope factor

# Wykres czynnikw
windows()
par(mfrow=c(3,1), cex = 0.7, bty="l")
plot(daty,L, main="level (L)", type="l", ylab="", lty=1, ylim=c(0,15))
plot(daty,C, main="curvature (C)", type="l", ylab="", lty=1, ylim=c(-8,8))
lines(daty,rep(0,T), type="l", ylab="", lty=1)
plot(daty,S, main="slope (S)", type="l", ylab="", lty=1, ylim=c(-5,5))
lines(daty,rep(0,T), type="l", ylab="", lty=1)

# 3. Prognoza dla stop z modeli klasy Diebold-Li
########################################

# horyzont prognozy
datyF <- seq(from=daty[T]+365.25/12, to=as.Date("2017-12-31"), by=365.25/12)
h     <- length(datyF)
# termin zapadalnosci + adunki  
k     <- 3            
LT    <- 1			                       
MT    <- (1-exp(-lam*k))/(lam*k) - exp(-lam*k);  
ST    <- (1-exp(-lam*k))/(lam*k)
Stopy <- YC[,which(tau==k)]  # przeszle dane


# 3a. Diebold-Li ARMA / VAR / BVAR    
########################################
# wybierz model (1 - ARMA, 2 - VAR, 3 - BVAR)
mdl = 1

# prognoza dla czynnikow z modelu ARMA(1,1)
if(mdl==1){
  require(forecast)
	mdlSpec <- c(1,0,1)
	armaL <- arima(L, mdlSpec) 
	armaC <- arima(C, mdlSpec)
	armaS <- arima(S, mdlSpec)
  fctL  <- forecast::forecast(armaL,h)$mean
	fctC  <- forecast::forecast(armaC, h)$mean
	fctS  <- forecast::forecast(armaS, h)$mean
} 

# prognoza dla czynnikow z modelu VAR(2)
if(mdl==2){
	require(vars)
	    varE     <- VAR(beta, 2)	
      fctBeta  <- predict(varE, n.ahead=h)
      fctL     <- fctBeta$fcst$LT[,"fcst"]
	    fctC     <- fctBeta$fcst$MT[,"fcst"]
      fctS     <- fctBeta$fcst$ST[,"fcst"]
	} 

# prognoza z modelu BVAR(2) - Minnesota prior
# set XiBeta to 0 for very tight prior, or to infty for very loose prior
if(mdl==3){
	require(BMR)
  bvarE     <- BVARW(beta,cores=1,coefprior=c(1,1,1),p=2,constant=TRUE, irf.periods=20,keep=1000,burnin=100,
                     XiBeta=0.1,XiSigma=1,gamma=NULL)
  #bvarE <- BVARM(beta,coefprior=c(1,1,1),p=2,constant=TRUE,irf.periods=20, 
  #            keep=10000,burnin=1000,VType=1,HP1=0.1,HP2=0.1,HP3=0.1)
  # coefprior - vector of length m prior for own first-lags
  # p 		- the number of lags 
  # keep 	- the number of Gibbs sampling replications to keep 
  # burnin 	- burn-in length for the Gibbs sampler
  # VType    	- type of prior 
  # HP1 - tightness for own lags
  # HP2 - tightness for other lags
  # HP3 - tightness for constant  
  
  fctBeta  <- forecast(bvarE,periods=h,plot=FALSE)
  fctL     <- fctBeta$PointForecast[,1]
  fctC     <- fctBeta$PointForecast[,2]
  fctS     <- fctBeta$PointForecast[,3]
  
} 

# Prognoza dla stop
Prognoza <- fctL*LT + fctC*MT + fctS*ST

# wykres prognozy
par(mfrow=c(1,1), cex = 0.7, bty="l")
plot(daty, Stopy, type="l", lty=1, col='black', xlim=as.Date(c("2005-01-01","2017-12-31")), ylim=c(0,6), ylab="") 
lines(datyF,Prognoza, type="l", lty=2, col='red')
title("Prognoza stp proc. z modelu Diebold Li")

# 3b. Prognoza dla stop z modelu: Diebol-Li VARmacro   
####################################################
# Korelacje ze zmiennymi makro   

x <- cbind(beta,MV); cor(x)
infl = MV[,1]
bezr = MV[,2]

# Factors vs inflation/unemployment
par(mfrow=c(1,2), cex = 0.7, bty="l")
plot(daty, L-mean(L), type="l", lty=1, col='black', ylab="",ylim=c(-5,11))
lines(daty, infl-mean(infl), type="l", lty=2, col='red')
title("level vs. inflacja")
plot(daty, S-mean(S), type="l", lty=1, col='black', ylab="",ylim=c(-4,6)) 
lines(daty, -bezr+mean(bezr), type="l", lty=2, col='red')
title("slope vs. bezrobocie")

# wybierz model (1 - VAR, 2 - BVAR)
mdl = 1


# prognoza dla czynnikow z modelu ARMA(1,1)
if(mdl==1){
	require(vars)
      varE     <- VAR(x, 2)	
      fctBeta  <- predict(varE, n.ahead=h)
      fctL     <- fctBeta$fcst$LT[,"fcst"]
	    fctC     <- fctBeta$fcst$MT[,"fcst"]
      fctS     <- fctBeta$fcst$ST[,"fcst"]
	} 

if(mdl==2){
	require(BMR)
  bvarE     <- BVARW(x,cores=1,coefprior=c(1,1,1,1,1),p=2,constant=TRUE, irf.periods=20,keep=1000,burnin=100,
                     XiBeta=0.01,XiSigma=1,gamma=NULL)
  #bvarE <- BVARM(x,coefprior=c(1,1,1,1,1),p=2,constant=TRUE,irf.periods=20, 
  #               keep=10000,burnin=1000,VType=1,HP1=0.1,HP2=0.1,HP3=0.1)
  fctBeta  <- forecast(bvarE,periods=h,plot=FALSE)
  fctL     <- fctBeta$PointForecast[,1]
  fctC     <- fctBeta$PointForecast[,2]
  fctS     <- fctBeta$PointForecast[,3]
	} 

# Prognoza dla stop
Prognoza <- fctL*LT + fctC*MT + fctS*ST

# wykres prognozy
par(mfrow=c(1,1), cex = 0.7, bty="l", pin=c(3,3))
plot(daty, Stopy, type="l", lty=1, col='black', xlim=as.Date(c("2005-01-01","2017-12-31")), ylim=c(0,6), ylab="") 
lines(datyF,Prognoza, type="l", lty=2, col='red')
title("Prognoza stp proc. z modelu Diebold Li - Macro endo")


# 3c. Warunkowa prognoza: Diebol-Li VARX macro   
####################################################

# scenariusz makroekonomiczny (random walk)
MVfct <- MV[T,]
for (i in 2:h) {
  MVfct <- rbind(MVfct,MV[T,])
}  
rownames(MVfct) <- datyF

# prognoza dla z modelu VAR(2)

	require(vars)
      varE     <- VAR(beta,exogen = MV, lag.max = 2)	
      fctBeta  <- predict(varE, n.ahead=h, dumvar = MVfct)
	    fctL     <- fctBeta$fcst$LT[,"fcst"]
	    fctC     <- fctBeta$fcst$MT[,"fcst"]
      fctS     <- fctBeta$fcst$ST[,"fcst"]
	

# Prognoza dla stop
Prognoza <- fctL*LT + fctC*MT + fctS*ST

# wykres prognozy
par(mfrow=c(1,1), cex = 0.7, bty="l", pin=c(3,3))
plot(daty, Stopy, type="l", lty=1, col='black', xlim=as.Date(c("2005-01-01","2017-12-31")), ylim=c(0,6), ylab="") 
lines(datyF,Prognoza, type="l", lty=2, col='red')
title("Prognoza stp proc. z modelu Diebold Li - macro exo")


