#####################################################################
# Calculates forecast errors                                        #
# Example:                                                          #                       
# err   <- FctErr(y, fct                                            #
# err   - fcst err. for y  [matrix of dimension (T-T1)xh]           #
#                                                                   #
# y     - data                                                      #                      
# fct   - matrix with forecasts [matrix of dimension (T-T1)xh]      #
#####################################################################

FctErr <- function(y, fct){

      Tf    <- nrow(fct)
      h     <- ncol(fct)
      junk  <- tail(y,Tf)
      junk  <- c(junk, rep(NA,h))
      act   <- matrix(NA,Tf,h)
      
      for (n in 1:Tf){
        act[n,]      <- junk[n:(n+h-1)]
      }
      
      # forecast errors
      err   <- act - fct
      
      return(list(err = err, act = act)) 
}

#####################################################################
# Calculates forecast error Statistics                              #
# Example:                                                          #                       
# stat <- FcstErrStats(err, balanced=TRUE)                          #
# stat$ME, stat$MAE, stat$RMSE                                      # 
#####################################################################

FctErrStats <- function(err){
    
    ME    <- colMeans(err, na.rm = T)
    MAE   <- colMeans(abs(err), na.rm = T)
    RMSE  <- sqrt(colMeans(err^2, na.rm = T))
    
    return(list(ME=ME, MAE=MAE, RMSE=RMSE))
}

#####################################################################
# Calculates DM test (two-sided)                                    #
# Example:                                                          #                       
# stat <- DMstat(err1, err2)                                        #
# DMstat                                                            # 
#####################################################################

DMstat <- function(err1, err2, type="two.sided", power=2){
  
  require(forecast)
  
  h       <- ncol(errRW)
  DMstat  <- matrix(NA,1,h)
  
  for (n in 1:h){
    e1 <- na.omit(err1[,n])
    e2 <- na.omit(err2[,n])
    
    junk         <- dm.test(e1, e2, alternative=type, h=n, power=power)
    DMstat[1,n]  <- junk$p.value
    
  }
  
  return(DMstat)
}


#dm.test(e1, e2, alternative=c("two.sided","less","greater"),h=1, power=2)


